#include "llvm/IR/Value.h"
#include <iostream>
#include <memory>
#include <sys/types.h>
#include <vector>

class ASTExpression {
public:
    virtual ~ASTExpression() {}

    virtual llvm::Value* codegen() = 0;
};

class ASTIntegerExpression : public ASTExpression {
    int value;

public:
    ASTIntegerExpression(int32_t value)
        : value(value)
    {
    }

    llvm::Value* codegen() override;
};

class ASTReturnExpression : ASTExpression {
    std::unique_ptr<ASTExpression> val;

public:
    ASTReturnExpression(std::unique_ptr<ASTExpression> val)
        : val(std::move(val))
    {
    }

    llvm::Value* codegen() override;
};

class ASTVariableExpression : public ASTExpression {
    std::string name;

public:
    ASTVariableExpression(const std::string& name)
        : name(name)
    {
    }

    llvm::Value* codegen() override;
};

class ASTVariableDeclarationExpression : public ASTExpression {
    std::string name;
    std::unique_ptr<ASTIntegerExpression> val;

public:
    ASTVariableDeclarationExpression(const std::string& name, std::unique_ptr<ASTIntegerExpression> val)
        : name(name)
        , val(std::move(val))
    {
    }

    llvm::Value* codegen() override;
};

/// TODO future work:

/// Prototype aka signature of a function
//class ASTFunctionPrototype {
//std::string name;
//std::vector<std::string> args;
//
//public:
//ASTFunctionPrototype(const std::string& name, std::vector<std::string> args)
//: name(name)
//, args(args)
//{
//}
//
//const std::string& get_name() const { return name; }
//};
//
//class ASTFunction {
//std::unique_ptr<ASTFunctionPrototype> prototype;
//std::unique_ptr<ASTExpression> body;
//
//public:
//ASTFunction(std::unique_ptr<ASTFunctionPrototype> prototype, std::unique_ptr<ASTExpression> body)
//: prototype(std::move(prototype))
//, body(std::move(body))
//{
//}
//};
